import argparse
import os

from .brainsegpicture import BrainSegPicture
from .brainsegnifti import BrainSegNifti


def parsercli():
    parser = argparse.ArgumentParser(prog='brainseg')

    parser.add_argument('filepath', type=str, help='Path of file to segment')

    parser.add_argument('-s', '--segment', type=str, help='Segments using provided SavedModel (Filepath required)')
    parser.add_argument('--segment-precision', type=int, choices=[32, 64], help='Float32 or float64')

    parser.add_argument('--info', '-i', action='store_true')

    parser.add_argument('--output', '-o', type=str, default='segmentednifti.nii.gz', help='Output filepath')
    parser.add_argument('--output-channels', type=int, choices=[1, 3], help='Number of channel in output image',
                        default=1)

    args = parser.parse_args()

    # Format parsing
    supported_image_ext = ['.png', '.jpg', '.jpeg']
    supported_nifti_ext = ['.nii', '.nii.gz', '.gz']
    _, extension = os.path.splitext(args.filepath)
    if extension.lower() in supported_image_ext:
        data = BrainSegPicture(args.filepath, extension)
        brainsegpicture_parser(args, data)
    elif extension.lower() in supported_nifti_ext:
        data = BrainSegNifti(args.filepath, args.segment_precision)
        nifti_parser(args, data)
    else:
        print(f'Input file format not supported. \nPlease use the following formats: {supported_image_ext}')


def brainsegpicture_parser(args, data):
    if args.info:
        data.information()
    if args.output:
        data.save_image(args.output, args.output_channels)


def nifti_parser(args, data):
    if args.segment:
        data.segment_nifti(args.segment)
    if args.output:
        data.write_nifti(args.output)


def main():
    parsercli()



import os
import numpy as np
import tensorflow as tf
import SimpleITK as sitk

from .brainsegnifti_helper import data_model_preprocess, resize_256


class BrainSegNifti:

    def __init__(self, filepath: str, precision: int):
        dtype = sitk.sitkFloat32 if precision == 32 else sitk.sitkFloat64

        sitk_image = sitk.ReadImage(filepath, dtype)

        self.nifti_array = sitk.GetArrayFromImage(sitk_image)
        self.file_name = os.path.basename(filepath)
        self.filepath = filepath
        self.reader = sitk.ImageFileReader()

    def information(self):
        print(f'--{self.file_name}--')
        print(f'Shape: {self.nifti_array.shape}')

        self.reader.SetFileName(self.filepath)
        self.reader.LoadPrivateTagsOn()

        self.reader.ReadImageInformation()

        for k in self.reader.GetMetaDataKeys():
            v = self.reader.GetMetaData(k)
            if not (v == '0' or v == ''):
                print(f"({k}) == \"{v}\"")

    def segment_nifti(self, model_path):
        model = tf.keras.models.load_model(model_path)

        data = data_model_preprocess(self.nifti_array, self.reader)
        self.segmented_array = []

        counter = 1
        for slice in data:
            pred_image = model.predict(slice[np.newaxis, :])

            pred_image = np.squeeze(pred_image, axis=0)
            pred_image = np.squeeze(pred_image, axis=-1)

            pred_image[pred_image > 0] = 1
            pred_image[pred_image < 0] = 0

            self.segmented_array.append(pred_image)

            counter += 1
            if counter % 50 == 0:
                print(counter, ' slices segmented')

        reshaped_array = resize_256(self.nifti_array, [0, 256, 256])

        for z in range(len(self.nifti_array)):
            for y in range(len(self.nifti_array[z])):
                for x in range(len(self.nifti_array[z][y])):
                    if self.segmented_array[z][y][x] == 1:
                        self.segmented_array[z][y][x] = reshaped_array[z][y][x]

    def write_nifti(self, out_filepath):
        if self.segmented_array:
            output = sitk.GetImageFromArray(self.segmented_array)
        else:
            output = sitk.GetImageFromArray(self.nifti_array)

        output_path = os.path.join(os.getcwd(), out_filepath)

        sitk.WriteImage(output, output_path)




from PIL import Image
import numpy as np


class BrainSegPicture:

    def __init__(self, filepath):
        self.image = Image.open(filepath)

        if self.image.mode == 'RGBA':
            image = self.image.convert('LA')
        elif self.image.mode == 'RGB':
            image = self.image.convert('L')
        else:
            image = self.image

        image = np.array(image)
        self.original_image_array = image

        if image.ndim != 2:
            for column in range(len(image)):
                for row in range(len(image[0])):
                    if image[column][row][1] == 0:
                        image[column][row] = 0

            image = image[..., :1]

        self.image_array = image

    def information(self):
        print(self.original_image_array.shape)
        print(self.image_array.shape)
        print(self.image.mode)

    def save_image(self, filepath, channels):
        output_image = Image.fromarray(self.image_array)
        if channels == 3:
            output_image = output_image.convert('RGB')
        elif channels == 1:
            output_image = output_image.convert('L')
        else:
            TypeError(f'Number of channels {channels} not supported in output format.')

        output_image.save(filepath)


import numpy as np
from niftiprocessing import niftihelper


def data_model_preprocess(data_array, reader):
    output_array = data_array
    output_array /= np.amax(output_array)

    try:
        output_array = niftihelper.align(output_array, reader)
    except RuntimeError:
        print('Unable to align due to invalid orientation xyz in metadata')

    output_array = resize_256(output_array, [0, 256, 256])

    return output_array


def resize_256(array, resize_shape: [int, int, int]) -> np.array:
    for dim in range(len(resize_shape)):
        if resize_shape[dim] == 0:
            resize_shape[dim] = array.shape[dim]
    shape_dif = np.subtract(resize_shape, array.shape)

    # Padding
    pad_list = []
    for dif in shape_dif:
        if dif <= 0:
            pad_list.append([0, 0])
        elif dif % 2 != 0:
            pad = int((dif - 1) / 2)
            pad_list.append([pad, pad + 1])
        else:
            pad = int(dif / 2)
            pad_list.append([pad, pad])

    output_array = np.pad(array, pad_list, mode='constant', constant_values=0.0)

    # Cropping
    cl = []  # crop_list
    for dif in shape_dif:
        if dif >= 0:
            cl.append([0, 256])
        elif dif % 2 != 0:
            crop = abs(int((dif + 1) / 2))
            cl.append([crop, crop + 256])
        else:
            crop = abs(int(dif / 2))
            cl.append([crop, crop + 256])

    output_array = output_array[cl[0][0]:cl[0][1], cl[1][0]:cl[1][1], cl[2][0]:cl[2][1]]

    return output_array